package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateInput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateOutput;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 日用电趋势计算Cloud模块
 *
 * @author ZWT
 * @date 2024/05/22 17:17
 */
@XService
public class DailyElectricityTrendCloudServiceImpl extends SpaceOptimizeBaseService implements IDailyElectricityTrendCloudService {

    /**
     * 日用电趋势计算
     * <p>
     * (1)取昨日极短期间开优化结果
     * (2)绿电阶段开井时间：每15分钟判段光伏出力>运行功率（考虑开井顺序,功率累加） 并且当前时段开井，则算坐开井时段(0.25小时)，全天累加
     * (3)单井绿电用电：运行功率*绿电阶段开井时间
     * (4)市电用电:总用电-光伏用电
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult dailyElectricityTrendJob(XContext context) {
        DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        //查每15分钟的有功功率
        Map<String, List<GetThirdActivePowerOutput>> powerMap = this.getAvgPhotovoltaicPower(context, yesterday);
        //查昨日间开优化结果
        Map<String, List<SpaceOptimizeShortDurationView>> lineMap = this.getOptimizationResult(context, yesterday);
        //查昨日日累计发电
        Map<String, BigDecimal> dailyUpdatePowerMap = this.getDailyUpdatePowerMap(context, yesterday);

        //遍历大的
        for (String stationName : powerMap.keySet()) {

        }
        List<DateTime> rangeToList = DateUtil.rangeToList(BusinessConstant.DATE_FLAG_BEGIN, BusinessConstant.DATE_FLAG, DateField.MINUTE, 15);
        return null;
    }

    /*------------------------------------- private -------------------------------------*/

    /**
     * 查询每日井场日累计数据
     *
     * @param context    上下文
     * @param createDate 创建日期
     * @return {@link Map }<{@link String }, {@link BigDecimal }>
     */
    private Map<String, BigDecimal> getDailyUpdatePowerMap(XContext context, Date createDate) {
        IThirdPowerCloudService service = context.getBean(IThirdPowerCloudService.class);
        XListResult<GetThirdDailyAccumulationUpdateOutput> result = service.queryDailyUpdatePower(context, GetThirdDailyAccumulationUpdateInput.builder()
                .createDate(createDate)
                .build());
        result.throwIfFail();
        Map<String, BigDecimal> powerMap;
        List<GetThirdDailyAccumulationUpdateOutput> powerList = result.getResult();
        if (CollUtil.isEmpty(powerList)) {
            powerMap = Collections.emptyMap();
        } else {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(GetThirdDailyAccumulationUpdateOutput::getStationName, GetThirdDailyAccumulationUpdateOutput::getPhotovoltaicPower));
        }
        return powerMap;
    }

    /**
     * 查每15分钟的有功功率
     *
     * @param context    上下文
     * @param createDate 创建日期
     * @return {@link Map }<{@link String }, {@link List }<{@link GetThirdActivePowerOutput }>>
     */
    private Map<String, List<GetThirdActivePowerOutput>> getAvgPhotovoltaicPower(XContext context, Date createDate) {
        IThirdPowerCloudService service = context.getBean(IThirdPowerCloudService.class);
        XListResult<GetThirdActivePowerOutput> result = service.queryAvgPhotovoltaicPower(context, GetThirdActivePowerInput.builder()
                .createDate(createDate)
                .build());
        result.throwIfFail();
        Map<String, List<GetThirdActivePowerOutput>> powerMap;
        List<GetThirdActivePowerOutput> powerList = result.getResult();
        if (CollUtil.isEmpty(powerList)) {
            powerMap = Collections.emptyMap();
        } else {
            powerMap = powerList.stream()
                    .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName));
        }
        return powerMap;
    }

    /**
     * 获取优化结果
     *
     * @param context      上下文
     * @param optimizeDate 优化日期
     * @return {@link Map }<{@link String }, {@link List }<{@link SpaceOptimizeShortDurationView }>>
     */
    private Map<String, List<SpaceOptimizeShortDurationView>> getOptimizationResult(XContext context, Date optimizeDate) {
        SpaceOptimizeShortDurationViewMapper shortMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationList = shortMapper.selectOptimizationResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(optimizeDate)
                .build());
        Map<String, List<SpaceOptimizeShortDurationView>> lineMap;
        if (CollUtil.isEmpty(durationList)) {
            lineMap = Collections.emptyMap();
        } else {
            lineMap = durationList.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName));
        }
        return lineMap;
    }
}
